package com.project.survey.logic.viewmodel

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.sqlite.db.SimpleSQLiteQuery
import blankj.utilcode.util.LogUtils
import com.bingce.data.database.PointDb
import com.bingce.data.database.PointStakeDb
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeRecord
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord
import com.project.survey.constants.Constants
import com.project.survey.constants.SPConstants
import com.project.survey.logic.bean.ListResult
import com.project.survey.model.ApprovalBean
import com.project.survey.model.ApprovalResponse
import com.project.survey.model.CoordinateSystemValue
import com.project.survey.model.InspectionReportBean
import com.project.survey.model.LoftingBean
import com.project.survey.model.LoftingPointBean
import com.project.survey.model.LoftingPointRecordEntry
import com.project.survey.model.MeasureBean
import com.project.survey.model.MeasureWorkStatusResult
import com.project.survey.model.PointEntry
import com.project.survey.model.ProjectBean
import com.project.survey.network.RetrofitClient
import com.project.survey.util.SPUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

class ProjectViewModel : BaseViewModel() {

    val api = RetrofitClient.createApiService()

    val projectResponse: LiveData<List<ProjectBean>>
        get() = _projectResponse
    private val _projectResponse = MutableLiveData<List<ProjectBean>>()

    /**
     * 获取用户在工作的所以项目的信息
     */
    fun fetchProjectList() {
        launch {
            val res = api.fetchProjectList(SPUtils.getString(SPConstants.PERSON_ID))
            if (res.success) {
                _projectResponse.postValue(res.data.map { it.values })
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val workTaskListResponse: LiveData<ApprovalResponse>
        get() = _workTaskListResponse
    private val _workTaskListResponse = MutableLiveData<ApprovalResponse>()

    /**
     *  获取流程审批列表
     *  @param end  0,未完结；1，已完结
     */
    fun fetchWorkTask(type: String, userName: String?, end: Int?, page: Int = 1) {
        launch {
            val res = api.fetchWorkTask(type = type, userName = userName, end = end, page = page)
            if (res.success) {
                _workTaskListResponse.postValue(res.data)
            }
        }
    }

    val loadFuncPermResponse: LiveData<ApprovalBean>
        get() = _loadFuncPermResponse
    private val _loadFuncPermResponse = MutableLiveData<ApprovalBean>()
    val loadFuncPermErrorResponse = MutableLiveData<String?>()

    /**
     * 加载功能权限
     */
    fun loadFuncPerm(bean: ApprovalBean) {
        launch {
            val res = api.loadFuncPerm(funcCode = bean.funcCode)
            if ("BD_KZW_ZY" == bean.funcCode) { //如果是专用控制网的，子功能也需要调用
                api.loadFuncPerm(funcCode = "BD_KZWZBCG_CHILD_ZY")
            }
            if ("BD_KZW" == bean.funcCode) { //如果是首级控制网的，子功能也需要调用
                api.loadFuncPerm(funcCode = "BD_KZWZBCG_CHILD")
            }
            if (res.success) {
                _loadFuncPermResponse.postValue(bean)
            } else {
                loadFuncPermErrorResponse.postValue(res.message)
            }
        }
    }

    val measureBeanListResponse: LiveData<ListResult<MeasureBean>>
        get() = _measureBeanListResponse
    private val _measureBeanListResponse = MutableLiveData<ListResult<MeasureBean>>()

    /**
     * 获取测量工作：测量计划(BD_CLJH)与工程计量(BD_CLGZ_GCJL)
     */
    fun fetchMeasureDataList(type: String, page: Int, keyword: String) {
        launch {
            val query = if (type == Constants.TYPE_BD_CLJH)
                "[{\"code\":\"SY_CURRENTTASK\",\"type\":\"=\",\"value\":\"计划发起人提交成果\"},{\"type\":\"and\",\"value\":[{\"code\":\"CLJH_LCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"CLJH_BDLSH\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_DWGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_FBGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_FXGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_DYGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"}]}]"
            else
                "[{\"code\":\"SY_CURRENTTASK\",\"type\":\"=\",\"value\":\"计划发起人提交成果\"},{\"type\":\"and\",\"value\":[{\"code\":\"GCJL_LCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"GCJL_BDLSH\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_DWGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_FBGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_FXGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"BD_FBFXPZ_DYGCMC\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"}]}]"


            val res = api.fetchMeasureDataList(
                tableCode = type,
                funcCode = if (type == Constants.TYPE_BD_CLJH) Constants.TYPE_BD_CLJH else "${Constants.TYPE_BD_CLGZ_GCJL}_ZB",
                j_query = query,
                page = page
            )
            if (res.success) {
                _measureBeanListResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val updateMeasureWorkStatusResponse: LiveData<MeasureWorkStatusResult>
        get() = _updateMeasureWorkStatusResponse
    private val _updateMeasureWorkStatusResponse = MutableLiveData<MeasureWorkStatusResult>()

    /**
     * 修改测量工作状态
     */
    fun updateMeasureWorkStatus(workId: String, workType: String, status: String) {
        launch {
            val res =
                api.updateMeasureWorkStatus(workId = workId, workType = workType, status = status)
            if (res.success) {
                _updateMeasureWorkStatusResponse.postValue(
                    MeasureWorkStatusResult(
                        workId,
                        status,
                        res.data
                    )
                )
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val uploadPointEntryResponse: LiveData<String>
        get() = _uploadPointEntryResponse
    private val _uploadPointEntryResponse = MutableLiveData<String>()

    /**
     * 测点记录录入
     */
    fun uploadPointEntry(pointRecord: PointRecord) {
        launch {
            val res = api.uploadPointEntry(requestBody = arrayListOf(PointEntry(pointRecord)))
            if (res.success) {
                // 上传成功，更新数据库里状态
                PointDb.getInstance().save(pointRecord.apply {
                    uploaded = true
                })
                _uploadPointEntryResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    /**
     * 批量上传 之前未上传或上传失败的点测量记录
     */
    fun uploadPointEntry() {
        launch {
            // 找出当前项目未上传的点
            val sqlString =
                "SELECT * FROM Point WHERE project_id=='${SPUtils.getString(SPConstants.PROJECT_ID)}' AND uploaded!=1 AND BC_DELETE != 1"
            val pointRecords = PointDb.getInstance().rawQueryListData(SimpleSQLiteQuery(sqlString))
            if (pointRecords.isNullOrEmpty()) {
                LogUtils.d("测点上传------没有找到未上传的点测量记录")
                return@launch
            }
            LogUtils.d("测点上传------有未上传的点测量记录")
            val res = api.uploadPointEntry(requestBody = pointRecords.map { PointEntry(it) })
            if (res.success) {
                // 上传成功，更新数据库里状态
                pointRecords.forEach { item -> item.uploaded = true }
                PointDb.getInstance().save(pointRecords)
                LogUtils.d("测点上传------上传点测量记录--成功")
            } else {
                LogUtils.d("测点上传------上传点测量记录--失败：${res.message}")
            }
        }
    }

    val createInspectionReportResponse: LiveData<InspectionReportBean>
        get() = _createInspectionReportResponse
    private val _createInspectionReportResponse = MutableLiveData<InspectionReportBean>()

    /**
     * 生成报验单
     */
    fun createInspectionReport(workId: String) {
        launch {
            val res = api.createInspectionReport(workId = workId)
            if (res.success) {
                _createInspectionReportResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val loftingBeanListResponse: LiveData<ListResult<LoftingBean>>
        get() = _loftingBeanListResponse
    private val _loftingBeanListResponse = MutableLiveData<ListResult<LoftingBean>>()

    /**
     * 获取放样任务列表
     */
    fun fetchLoftingList(keyword: String) {
        launch {
            val query =
                "[{\"code\":\"ZYLB_ZYLBZT_NAME\",\"type\":\"=\",\"value\":\"执行中\"},{\"type\":\"and\",\"value\":[{\"code\":\"ZYLB_ZY_NAME\",\"type\":\"like\",\"value\":\"${keyword}\",\"cn\":\"or\"},{\"code\":\"ZYLB_CLY\",\"type\":\"like\",\"value\":\"${keyword}\"}]}]"
            val res = api.fetchLoftingList(j_qurey = query)
            if (res.success) {
                _loftingBeanListResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val loftingPointListResponse: LiveData<List<LoftingPointBean>>
        get() = _loftingPointListResponse
    private val _loftingPointListResponse = MutableLiveData<List<LoftingPointBean>>()

    /**
     * 获取放样点
     */
    fun fetchLoftingPointList(workId: String) {
        launch {
            val res = api.fetchLoftingPointList(workId = workId)
            if (res.success) {
                _loftingPointListResponse.postValue(res.data.map { it.values })
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val uploadLoftingPointRecordResponse: LiveData<String>
        get() = _uploadLoftingPointRecordResponse
    private val _uploadLoftingPointRecordResponse = MutableLiveData<String>()


    /**
     * 上传放样测点记录
     */
    fun uploadLoftingPointRecord(record: PointStakeRecord) {
        launch {
            val res = api.uploadLoftingPointRecord(
                requestBody = arrayListOf(
                    LoftingPointRecordEntry(record)
                )
            )
            if (res.success) {
                // 上传成功
                PointStakeDb.getInstance().save(record.apply {
                    uploaded = true
                })
                _uploadLoftingPointRecordResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    /**
     * 批量上传 之前未上传或上传失败的放样点记录
     */
    fun uploadLoftingPointRecord() {
        launch {
            // 找到未上传的列表
            val sqlStringLofting =
                "SELECT * FROM Point_Stake WHERE project_id=='${SPUtils.getString(SPConstants.PROJECT_ID)}' AND uploaded!=1 AND BC_DELETE != 1"
            val pointStakeRecords =
                PointStakeDb.getInstance().rawQueryListData(SimpleSQLiteQuery(sqlStringLofting))
            if (pointStakeRecords.isNullOrEmpty()) {
                LogUtils.d("测点上传------没有找到未上传的放样点记录")
                return@launch
            }
            LogUtils.d("测点上传------有未上传的放样点记录")
            val res = api.uploadLoftingPointRecord(requestBody = pointStakeRecords.map {
                LoftingPointRecordEntry(it)
            })
            if (res.success) {
                // 上传成功
                pointStakeRecords.forEach { item -> item.uploaded = true }
                PointStakeDb.getInstance().save(pointStakeRecords)
                LogUtils.d("测点上传------上传放样点记录--成功")
            } else {
                LogUtils.d("测点上传------上传放样点记录--失败：${res.message}")
            }
        }
    }

    /**
     * 拉取坐标
     */
    fun fetchCoordinateSystem(onSuccess: (CoordinateSystemValue) -> Unit, onFailure: (String) -> Unit) {
        launch {
            val res = api.downloadCoordinateSystem()
            if (res.success) {
                if (res.data.isNotEmpty())
                    withContext(Dispatchers.Main) {
                        onSuccess(res.data[0].values)
                    }
            } else {
                withContext(Dispatchers.Main) {
                    onFailure(res.message)
                }
            }
        }
    }

}