package com.project.survey.network

import com.bingce.http.HttpsTrustUtil
import com.project.survey.BuildConfig
import com.project.survey.constants.SPConstants
import com.project.survey.util.DESUtil
import com.project.survey.util.SPUtils
import okhttp3.FormBody
import okhttp3.HttpUrl.Companion.toHttpUrlOrNull
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

object RetrofitClient {

    fun createApiService(): Api {
        return createService<Api>()
    }

    inline fun <reified T> createService(): T {
        return createRetrofit().create(T::class.java)
    }


    fun createRetrofit(): Retrofit {
        return Retrofit.Builder()
            .baseUrl(BuildConfig.BASE_URL)
            .client(createOkHttpClient())
            .addConverterFactory(GsonConverterFactory.create())
            .addCallAdapterFactory(RxJava2CallAdapterFactory.create()) //retrofit默认支持Call返回类型，改为支持Observable返回类型
            .build()
    }

    private fun createOkHttpClient(): OkHttpClient {
        return OkHttpClient.Builder()
            .connectTimeout(60L, TimeUnit.SECONDS)
            .readTimeout(60L, TimeUnit.SECONDS)
            .writeTimeout(60L, TimeUnit.SECONDS)
            .addInterceptor { chain ->
                //获取request
                val request = chain.request()
                //从request中获取原有的HttpUrl实例oldHttpUrl
                val oldHttpUrl = request.url
                //获取request的创建者builder
                val builder = request.newBuilder()
                //从request中获取headers
                val headerValues = request.headers("usePD")
                if (headerValues.isNotEmpty()){
                    builder.removeHeader("pd")
                } else {
                    builder.addHeader("pd", "jecloud-gcclzhglpt")
                }
                val token = SPUtils.getString(SPConstants.TOKEN)
                builder.removeHeader("Authorization")
                if (token.isNotBlank()) {
                    builder.addHeader("Authorization", token)
                }

                val baseURL = BuildConfig.BASE_URL.toHttpUrlOrNull()!!
                //重建新的HttpUrl，需要重新设置的url部分
                val newHttpUrl = oldHttpUrl.newBuilder()
                    .scheme(baseURL.scheme) //http协议如：http或者https
                    .host(baseURL.host) //主机地址
                    .port(baseURL.port) //端口
                    .build()
                //获取处理后的新newRequest
                val newRequest = builder.url(newHttpUrl).build()
                return@addInterceptor chain.proceed(newRequest)
            }
            .addInterceptor { chain ->
                val originalRequest = chain.request()
                if (originalRequest.method == "GET") {
                    val originalUrl = originalRequest.url
                    // 创建一个新的 HttpUrl.Builder
                    val urlBuilder = originalUrl.newBuilder()
                    // 定义一个map用来存储参数
                    val params = mutableMapOf<String, Any>()
                    // 遍历原始 URL 的参数
                    for (name in originalUrl.queryParameterNames) {
                        val value = originalUrl.queryParameter(name)
                        // 放入map中
                        params[name] = value ?: ""
                        // 移除原来的参数
                        urlBuilder.removeAllQueryParameters(name)
                        // 加密参数名
                    }
                    // 对参数名和参数值进行加密
                    val encryptedMap = DESUtil.toParamsEncrypt(params)
                    encryptedMap.forEach { (key, value) ->
                        if (key != "params-keys") {
                            urlBuilder.setQueryParameter(key, value.toString())
                        }
                    }
                    val newRequest = originalRequest.newBuilder()
                        .url(urlBuilder.build())
                        .header("params-keys", encryptedMap["params-keys"].toString())
                        .build()
                    return@addInterceptor chain.proceed(newRequest)
                }
                if (originalRequest.method == "POST" && originalRequest.body != null && originalRequest.body is FormBody) {
                    val formBody = originalRequest.body as FormBody
                    // 构建一个新的 FormBody.Builder
                    val formBodyBuilder = FormBody.Builder()
                    // 定义一个map用来存储参数
                    val params = mutableMapOf<String, Any>()
                    for (i in 0 until formBody.size) {
                        params[formBody.name(i)] = formBody.value(i)
                    }
                    // 对参数名和参数值进行加密
                    val encryptedMap = DESUtil.toParamsEncrypt(params)
                    encryptedMap.forEach { (key, value) ->
                        if (key != "params-keys") {
                            formBodyBuilder.add(key, value.toString())
                        }
                    }
                    val newRequest = originalRequest.newBuilder()
                        .method(originalRequest.method, formBodyBuilder.build())
                        .header("params-keys", encryptedMap["params-keys"].toString())
                        .build()
                    return@addInterceptor chain.proceed(newRequest)
                }
                return@addInterceptor chain.proceed(originalRequest)
            }
            .addInterceptor(HttpLoggingInterceptor().apply {
                level = if (BuildConfig.DEBUG)
                    HttpLoggingInterceptor.Level.BODY
                else
                    HttpLoggingInterceptor.Level.NONE
            })
            .sslSocketFactory(
                HttpsTrustUtil.createSSLSocketFactory(),
                HttpsTrustUtil.TrustAllManager()
            )
            .hostnameVerifier(HttpsTrustUtil.TrustAllHostnameVerifier())
            .build()
    }
}