package com.project.survey.ui.base

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.view.MenuItem
import android.view.WindowManager
import androidx.annotation.StringRes
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.appcompat.widget.Toolbar
import androidx.core.app.ActivityCompat
import androidx.viewbinding.ViewBinding
import blankj.utilcode.util.LogUtils
import com.bingce.coordlib.model.Blh
import com.bingce.coordlib.model.Coordinate
import com.bingce.device.Device
import com.bingce.device.Device.IDeviceRequestListener
import com.bingce.device.ui.manager.DeviceConnectManager
import com.bingce.device.ui.manager.DeviceRequestManager
import com.bingce.device.ui.manager.RtkManager
import com.bingce.device.ui.manager.TsAnglesRefreshManager
import com.bingce.device.ui.manager.TsBatteryRefreshManager
import com.bingce.device.ui.manager.TsChangeHrManager
import com.bingce.device.ui.manager.TsMeasCoordinateManager
import com.bingce.device.ui.manager.TsMeasSdAnglesManager
import com.bingce.totalstation.TsListener
import com.gyf.immersionbar.ImmersionBar
import com.project.survey.R
import com.project.survey.ui.MainActivity
import com.project.survey.ui.MainActivityUtils
import com.project.survey.widget.LoadingDialog
import org.polaric.colorful.Colorful

abstract class BaseBindingActivity<VB : ViewBinding> : AppCompatActivity() {

    private lateinit var _binding: VB
    protected val mBinding: VB
        get() = _binding

    private var toolBar: Toolbar? = null

    protected lateinit var mContext: Context
    abstract fun getBinding(): VB
    abstract fun initView()
    abstract fun initData()
    abstract fun initListener()

    val mainActivityUtils: MainActivityUtils = MainActivityUtils(this) { this }

    protected val deviceRequestManager: DeviceRequestManager = DeviceRequestManager(this, object: IDeviceRequestListener{
        override fun onDeviceRequestConnect() {
//            LogUtils.d("onDeviceRequestConnect")
            mainActivityUtils.onRequestConnect()
        }

        override fun onDeviceRequestDisconnect() {
//            LogUtils.d("onDeviceRequestDisconnect")
        }

        override fun onDeviceRequestSetupChat() {
//            LogUtils.d("onDeviceRequestSetupChat")
        }
    })
    protected val deviceConnectManager: DeviceConnectManager = DeviceConnectManager(this, object : Device.IDeviceConnectListener{
        override fun onDeviceConnectSucceed() {
//            LogUtils.d("onDeviceConnectSucceed")
            mainActivityUtils.onDeviceConnectSuccess()
        }

        override fun onDeviceConnectLost() {
//            LogUtils.d("onDeviceConnectLost")
            mainActivityUtils.onDeviceConnectLost()
        }

        override fun onDeviceConnectFailed() {
//            LogUtils.d("onDeviceConnectFailed")
            mainActivityUtils.onDeviceConnectFailed()
        }

        override fun onDeviceConnectNeedOpenBT() {
//            LogUtils.d("onDeviceConnectNeedOpenBT")
            if (ActivityCompat.checkSelfPermission(mContext, Manifest.permission.BLUETOOTH_CONNECT ) == PackageManager.PERMISSION_GRANTED) {
                val intent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
                startActivityForResult(intent, MainActivity.RC_OPEN_BT)
            }
        }

    })
    protected val rtkManager: RtkManager = RtkManager(this, object : RtkManager.IRtkListener {
        override fun onRtkPositionReceived(
            p0: Double,
            p1: Blh?,
            p2: Blh?,
            p3: Coordinate?,
            p4: Double,
            p5: Double,
            p6: Double
        ) {
//            LogUtils.d("onRtkPositionReceived:${p0};${p1};${p2};${p3};${p4};${p5};${p6};")
            mainActivityUtils.checkRtkLocation()
        }

        override fun onRtkStatusChanged() {
//            LogUtils.d("onRtkStatusChanged")
        }

        override fun onRtkError(p0: String?) {
//            LogUtils.d("onRtkError:${p0}")
        }

        override fun onRtkPoleHeightChanged() {
//            LogUtils.d("onRtkPoleHeightChanged")
        }

        override fun onRtkTiltSwitched(p0: Boolean) {
//            LogUtils.d("onRtkTiltSwitched:${p0}")
        }

        override fun onSatelliteSkyViewRefresh() {
//            LogUtils.d("onSatelliteSkyViewRefresh")
        }

    })
    protected val tsMeasCoordinateManager: TsMeasCoordinateManager =
        TsMeasCoordinateManager(this, object : TsListener.MeasCoordinateListener{
            override fun onMeasCoordinateSuccess(p0: Coordinate?) {
//                LogUtils.d("onMeasCoordinateSuccess:${p0}")
            }

            override fun onMeasCoordinateFailed(p0: String?, p1: String?) {
//                LogUtils.d("onMeasCoordinateFailed:${p0};${p1}")
            }
        })
    protected val tsMeasSdAnglesManager: TsMeasSdAnglesManager = TsMeasSdAnglesManager(this, object : TsListener.MeasSdAnglesListener{
        override fun onMeasSdAnglesSuccess(
            p0: Double,
            p1: Double,
            p2: Double,
            p3: Double,
            p4: Double,
            p5: Double,
            p6: Double
        ) {
//            LogUtils.d("onMeasSdAnglesSuccess:${p0};${p1};${p2};${p3};${p4};${p5};${p6}")
        }

        override fun onMeasSdAnglesFailed(p0: String?, p1: String?) {
//            LogUtils.d("onMeasSdAnglesFailed:${p0};${p1}")
        }
    })
    protected val tsAnglesRefreshManager: TsAnglesRefreshManager =
        TsAnglesRefreshManager(this, object : TsListener.AnglesRefreshListener{
            override fun onAnglesRefresh(p0: Double, p1: Double) {
//                LogUtils.d("onAnglesRefresh:${p0};${p1}")
            }
        })
    protected val tsBatteryRefreshManager: TsBatteryRefreshManager =
        TsBatteryRefreshManager(this) { }
    protected val tsChangeHrManager: TsChangeHrManager = TsChangeHrManager(this, object : TsListener.ChangeHrListener{
        override fun onChangeHrSuccess(p0: Double) {
//            LogUtils.d("onChangeHrSuccess:${p0}")
        }

        override fun onChangeHrFailed(p0: String?) {
//            LogUtils.d("onChangeHrSuccess:${p0}")
        }

    })

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mContext = this
        _binding = getBinding()
        setContentView(mBinding.root)
//        //设置背景色
//        mBinding.root.setBackgroundColor(Tools.getColor(R.color.bg_content))

//        initToolBar()
        initView()
        initData()
        initListener()
    }

    private fun initTheme() {
        setTheme(Colorful.getThemeDelegate().styleResBase)
        getTheme().applyStyle(Colorful.getThemeDelegate().styleResPrimary, true)
        getTheme().applyStyle(Colorful.getThemeDelegate().styleResAccent, true)
        delegate.localNightMode = AppCompatDelegate.MODE_NIGHT_YES
//        if (app.isThemeDark) {
//            delegate.localNightMode = AppCompatDelegate.MODE_NIGHT_YES
//        } else {
//            delegate.localNightMode = AppCompatDelegate.MODE_NIGHT_NO
//        }
        window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
    }

    protected open fun initToolBar() {
        toolBar = findViewById(R.id.toolbar)
        setSupportActionBar(toolBar)
        if (supportActionBar != null) supportActionBar!!.setDisplayHomeAsUpEnabled(true)
    }

    protected fun setToolbarTitle(text: String?) {
        toolBar?.setTitle(text)
    }

    protected fun setToolbarTitle(@StringRes text: Int) {
        toolBar?.setTitle(text)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            if (isFinishAtOnce())
                finish()
        }
        return super.onOptionsItemSelected(item)
    }

    protected open fun isFinishAtOnce(): Boolean {
        return true
    }

    protected fun immersionToolbar(toolbar: Toolbar, titleResId: Int) {
        immersionToolbar(toolbar, getString(titleResId))
    }

    protected fun immersionToolbar(toolbar: Toolbar, title: String? = null) {
        ImmersionBar.with(this)
            .statusBarDarkFont(true)
            .titleBarMarginTop(toolbar)
            .init()
        setSupportActionBar(toolbar)
        toolbar.setBackgroundColor(0)
//        toolbar.setTitleTextColor(ContextCompat.getColor(this, R.color.text_color_main))
//        toolbar.setTitle(title)
        supportActionBar?.title = title
    }

    private var loadingDialog: LoadingDialog? = null

    protected fun showLoadingDialog() {
        if (loadingDialog == null) {
            loadingDialog = LoadingDialog.Builder(this)
                .setCancelable(true)
                .setCancelOutside(true)
                .create()
        }
        loadingDialog?.show()
    }

    protected fun dismissLoadingDialog() {
        if (loadingDialog?.isShowing == true) {
            loadingDialog?.dismiss()
        }
    }
}