package com.project.survey.ui.instrument.setupstation.util;

import android.util.Log;

import androidx.annotation.WorkerThread;

import com.bingce.coordlib.enums.DatumEnum;
import com.bingce.coordlib.enums.EllipsoidEnum;
import com.bingce.coordlib.enums.GridEnum;
import com.bingce.coordlib.enums.HFittingEnum;
import com.bingce.coordlib.enums.ProjectionEnum;
import com.bingce.coordlib.model.Blh;
import com.bingce.coordlib.model.BlhCoordinatePair;
import com.bingce.coordlib.model.Coordinate;
import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.coordlib.util.CoordinateSystemUtil;
import com.bingce.coordlib.util.CoordinateSystemUtilOld;
import com.bingce.poi.excel.ExcelReadHelper;
import com.bingce.utils.Util;

import java.io.File;
import java.util.List;

import blankj.utilcode.util.Utils;

class _3CoordinatorCollectUtils {
    /**
     * 收集坐标系数据
     *
     * @return 成功返回CoordinateRecord实例（已经被存储到数据库中），失败返回null
     */
    @WorkerThread
    static CoordinateSystem collectCoordinatorOfProject(File projectFolder) {
        //先尝试收集.json
        CoordinateSystem coordinateSystem = CoordinateSystemUtilOld.importCoordSysJsonFile(projectFolder);
        //如果json坐标文件不存在，尝试读取excel
        if (coordinateSystem == null) {
            coordinateSystem = importCoordSysExcelFile(projectFolder);
        }
        if (coordinateSystem == null){
            coordinateSystem = CoordinateSystemUtil.importCoordSysStringFile(projectFolder);
        }
        //如果仍然没有配置文件，则尝试新建
        if (coordinateSystem == null) {
            coordinateSystem = CoordinateSystem.newCoordinateSystem(Utils.getApp());
        }
        return coordinateSystem;
    }


    /**
     * 导入excel版坐标系统
     *
     * @param prjFolder 工程文件夹路径
     * @return 坐标系统
     */
    private static CoordinateSystem importCoordSysExcelFile(File prjFolder) {
        File sysFile = new File(prjFolder, "coordinate_system.xls");
        if (!sysFile.exists() || sysFile.isDirectory()) {
            return null;
        }
        CoordinateSystem coordinateSystem = new CoordinateSystem();
        coordinateSystem.isPreset = false;
        int line = 0;
        try {
            List<List<List<String>>> importResult = ExcelReadHelper.excelReadBySheet(sysFile, ExcelReadHelper.FIRST_ROW_NOT_IGNORE);
            if (importResult == null) return coordinateSystem;
            if (importResult.isEmpty()) return coordinateSystem;

            line = 0;
            List<List<String>> sheetResult = importResult.get(0);
            if (sheetResult == null || sheetResult.isEmpty()) {
                Log.d("CoordinateSystemUtil", sysFile.getAbsolutePath() + "，导入失败，原因：导入excel为空");
                return coordinateSystem;
            }
            String sheetName = sheetResult.get(0).isEmpty() ? "default" : sheetResult.get(0).get(0);
            coordinateSystem.setName(sheetName);
            for (int j = 1; j < sheetResult.size(); j++) {
                List<String> row = sheetResult.get(j);

                line = j + 1;
                if (row.isEmpty() || ExcelReadHelper.isRowEmpty(row)) {
                    continue;
                }
                if (row.size() != 2 && row.size() != 7 && row.size() != 6 && row.size() != 9) {
                    Log.d("CoordinateSystemUtil", sysFile.getAbsolutePath() + "，导入失败，原因：第" + line + "行，元素个数必须为2,7,6,9");
                    break;
                }
                if (row.size() == 2) {
                    switch (row.get(0)) {
                        case "isPreset":
                            int preset = Integer.parseInt(row.get(1));
                            coordinateSystem.isPreset = preset == 1;
                            break;
                        case "rtkTransformMethod":
                        case "transMethod":
                            int method = Integer.parseInt(row.get(1));
                            if (method == 0) {
                                coordinateSystem.datum = DatumEnum.BURSA;
                                coordinateSystem.grid = GridEnum.NULL;
                                coordinateSystem.hFitting = HFittingEnum.NULL;
                            } else if (method == 1 || method == 2) {
                                coordinateSystem.datum = DatumEnum.NULL;
                                coordinateSystem.grid = GridEnum.FOUR_PARA;
                                coordinateSystem.hFitting = HFittingEnum.FIX;
                            }
                            break;
                        case "ellipsoidType":
                            int ellipsoid = Integer.parseInt(row.get(1));
                            switch (ellipsoid) {
                                case 0:
                                    coordinateSystem.setEllipsoid(EllipsoidEnum.WGS_84);
                                    break;
                                case 1:
                                    coordinateSystem.setEllipsoid(EllipsoidEnum.BEIJING_54);
                                    break;
                                case 2:
                                    coordinateSystem.setEllipsoid(EllipsoidEnum.XIAN_80);
                                    break;
                                default:
                                    coordinateSystem.setEllipsoid(EllipsoidEnum.CGCS_2000);
                                    break;
                            }
                            break;
                        case "rtkProjectType":
                        case "projectType":
                            int type = Integer.parseInt(row.get(1));
                            switch (type) {
                                case 2:
                                    coordinateSystem.setProjection(ProjectionEnum.UTM);
                                    break;
                                default:
                                    coordinateSystem.setProjection(ProjectionEnum.GAUSS);
                            }
                            break;
                        case "datumTransType":
                            int datumTransType = Integer.parseInt(row.get(1));
                            if (datumTransType < 0 || datumTransType > 1)
                                datumTransType = 0;
                            switch (datumTransType) {
                                case 1:
                                    coordinateSystem.datum = DatumEnum.BURSA;
                                    break;
                                default:
                                    coordinateSystem.datum = DatumEnum.NULL;
                            }
                            break;
                        case "planeTransType":
                            int planeTransType = Integer.parseInt(row.get(1));
                            switch (planeTransType) {
                                case 1:
                                    coordinateSystem.grid = GridEnum.FOUR_PARA;
                                    break;
                                default:
                                    coordinateSystem.grid = GridEnum.NULL;
                            }
                            break;
                        case "elvFitType":
                            int elvFitType = Integer.parseInt(row.get(1));
                            switch (elvFitType) {
                                case 1:
                                case 4:
                                    coordinateSystem.hFitting = HFittingEnum.FIX;
                                    break;
                                case 2:
                                    coordinateSystem.hFitting = HFittingEnum.PLANE;
                                    break;
                                case 3:
                                    coordinateSystem.hFitting = HFittingEnum.CURVE;
                                    break;
                                case 5:
                                    coordinateSystem.hFitting = HFittingEnum.TGO;
                                    break;
                                default:
                                    coordinateSystem.hFitting = HFittingEnum.NULL;
                            }
                            break;
                        case "longOfOrigin":
                        case "centLng":
                            coordinateSystem.centLng = Double.parseDouble(row.get(1));
                            break;
                        case "latOfOrigin":
                        case "baseLat":
                            coordinateSystem.baseLat = Double.parseDouble(row.get(1));
                            break;
                        case "projectionScale":
                        case "projScl":
                            coordinateSystem.projScl = Double.parseDouble(row.get(1));
                            break;
                        case "falseEasting":
                        case "falseE":
                            coordinateSystem.falseE = Double.parseDouble(row.get(1));
                            break;
                        case "falseNorthing":
                        case "falseN":
                            coordinateSystem.falseN = Double.parseDouble(row.get(1));
                            break;
                        case "projectionHeight":
                        case "projAlt":
                            coordinateSystem.projAlt = Double.parseDouble(row.get(1));
                            break;
                        case "sevenParDx":
                            coordinateSystem.sevenDx = Double.parseDouble(row.get(1));
                            break;
                        case "sevenParDy":
                            coordinateSystem.sevenDy = Double.parseDouble(row.get(1));
                            break;
                        case "sevenParDz":
                            coordinateSystem.sevenDz = Double.parseDouble(row.get(1));
                            break;
                        case "sevenParRx":
                            coordinateSystem.sevenRx = Double.parseDouble(row.get(1)) * 3600d;
                            break;
                        case "sevenParRy":
                            coordinateSystem.sevenRy = Double.parseDouble(row.get(1)) * 3600d;
                            break;
                        case "sevenParRz":
                            coordinateSystem.sevenRz = Double.parseDouble(row.get(1)) * 3600d;
                            break;
                        case "sevenParScl":
                            coordinateSystem.sevenPpm = (Double.parseDouble(row.get(1)) - 1) * 1000000d;
                            break;
                        case "fourParDx":
                            coordinateSystem.fourDx = Double.parseDouble(row.get(1));
                            break;
                        case "fourParDy":
                            coordinateSystem.fourDy = Double.parseDouble(row.get(1));
                            break;
                        case "fourParRot":
                            coordinateSystem.fourRot = Util.radianToDmsDouble(Math.toRadians(Double.parseDouble(row.get(1))));
                            break;
                        case "fourParScl":
                            coordinateSystem.fourScl = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA0":
                            coordinateSystem.hFitA0 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA1":
                            coordinateSystem.hFitA1 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA2":
                            coordinateSystem.hFitA2 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA3":
                            coordinateSystem.hFitA3 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA4":
                            coordinateSystem.hFitA4 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitA5":
                            coordinateSystem.hFitA5 = Double.parseDouble(row.get(1));
                            break;
                        case "hFitN":
                            coordinateSystem.hFitN = Double.parseDouble(row.get(1));
                            break;
                        case "hFitE":
                            coordinateSystem.hFitE = Double.parseDouble(row.get(1));
                            break;
                        case "hFitNorthOrigin":
                            coordinateSystem.hFitNorthOrigin = Double.parseDouble(row.get(1));
                            break;
                        case "hFitEastOrigin":
                            coordinateSystem.hFitEastOrigin = Double.parseDouble(row.get(1));
                            break;
                        case "hFitNorthSlope":
                            coordinateSystem.hFitNorthSlope = Double.parseDouble(row.get(1));
                            break;
                        case "hFitEastSlope":
                            coordinateSystem.hFitEastSlope = Double.parseDouble(row.get(1));
                            break;
                        case "hFitHdConstant":
                            coordinateSystem.hFitHdConstant = Double.parseDouble(row.get(1));
                            break;
                        case "correctX":
                            coordinateSystem.correctN = -1 * Double.parseDouble(row.get(1));
                            break;
                        case "correctY":
                            coordinateSystem.correctE = -1 * Double.parseDouble(row.get(1));
                            break;
                        case "correctZ":
                            coordinateSystem.correctH = -1 * Double.parseDouble(row.get(1));
                            break;
                        case "correctN":
                            coordinateSystem.correctN = Double.parseDouble(row.get(1));
                            break;
                        case "correctE":
                            coordinateSystem.correctE = Double.parseDouble(row.get(1));
                            break;
                        case "correctH":
                            coordinateSystem.correctH = Double.parseDouble(row.get(1));
                            break;
                    }
                } else if (row.size() == 6) {
                    coordinateSystem.getPairs().add(new BlhCoordinatePair("", new Blh(Double.parseDouble(row.get(0)), Double.parseDouble(row.get(1)), Double.parseDouble(row.get(2)))
                            , new Coordinate(Double.parseDouble(row.get(3)), Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)))));
                } else if (row.size() == 9) {
                    if ("BlhPointPair".equals(row.get(0).trim())) {
                        switch (Integer.parseInt(row.get(1))) {
                            case 47:
                                coordinateSystem.getPairs().add(new BlhCoordinatePair(row.get(2), new Blh(Double.parseDouble(row.get(3)), Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)))
                                        , new Coordinate(Double.parseDouble(row.get(6)), Double.parseDouble(row.get(7)), Double.parseDouble(row.get(8)))));
                                break;
                            case 51:
                                coordinateSystem.phonePair1 = new BlhCoordinatePair(row.get(2), new Blh(Double.parseDouble(row.get(3)), Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)))
                                        , new Coordinate(Double.parseDouble(row.get(6)), Double.parseDouble(row.get(7)), Double.parseDouble(row.get(8))));
                                break;
                            case 52:
                                coordinateSystem.phonePair2 = new BlhCoordinatePair(row.get(2), new Blh(Double.parseDouble(row.get(3)), Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)))
                                        , new Coordinate(Double.parseDouble(row.get(6)), Double.parseDouble(row.get(7)), Double.parseDouble(row.get(8))));
                                break;
                        }
                    }
                } else {
                    switch (Integer.parseInt(row.get(0))) {
                        case 4:
                        case 7:
                            coordinateSystem.getPairs().add(new BlhCoordinatePair("", new Blh(Math.toDegrees(Util.dmsStringToRadian(row.get(1), false)), Math.toDegrees(Util.dmsStringToRadian(row.get(2), false)), Double.parseDouble(row.get(3)))
                                    , new Coordinate(Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), Double.parseDouble(row.get(6)))));
                            break;
                        case 31:
                            coordinateSystem.phonePair1 = new BlhCoordinatePair("", new Blh(Math.toDegrees(Util.dmsStringToRadian(row.get(1), false)), Math.toDegrees(Util.dmsStringToRadian(row.get(2), false)), Double.parseDouble(row.get(3)))
                                    , new Coordinate(Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), Double.parseDouble(row.get(6))));
                            break;
                        case 32:
                            coordinateSystem.phonePair2 = new BlhCoordinatePair("", new Blh(Math.toDegrees(Util.dmsStringToRadian(row.get(1), false)), Math.toDegrees(Util.dmsStringToRadian(row.get(2), false)), Double.parseDouble(row.get(3)))
                                    , new Coordinate(Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), Double.parseDouble(row.get(6))));
                            break;
                        case 41:
                            coordinateSystem.phonePair1 = new BlhCoordinatePair("", new Blh(Double.parseDouble(row.get(1)), Double.parseDouble(row.get(2)), Double.parseDouble(row.get(3)))
                                    , new Coordinate(Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), Double.parseDouble(row.get(6))));
                            break;
                        case 42:
                            coordinateSystem.phonePair2 = new BlhCoordinatePair("", new Blh(Double.parseDouble(row.get(1)), Double.parseDouble(row.get(2)), Double.parseDouble(row.get(3)))
                                    , new Coordinate(Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), Double.parseDouble(row.get(6))));
                            break;
                    }
                }
            }

            Log.d("CoordinateSystemUtil", sysFile.getAbsolutePath() + "，导入成功");
        } catch (NumberFormatException ex) {
            Log.d("CoordinateSystemUtil", sysFile.getAbsolutePath() + "，导入失败，原因：第" + line + "行，发现非数字字符串");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return coordinateSystem;
    }
}
