package com.project.survey.ui.project

import android.view.Menu
import android.view.MenuItem
import androidx.activity.viewModels
import com.afollestad.materialdialogs.MaterialDialog
import com.bingce.coordlib.model.CoordinateSystem
import com.bingce.coordlib.util.CoordinateSystemUtil
import com.bingce.data.database.KeyValueDb
import com.bingce.data.database.ProjectDb
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord
import com.bingce.data.surveyor.designdata.project.ProjectRecord
import com.bingce.utils.ThreadPoolUtil
import com.project.survey.R
import com.project.survey.constants.EventConstants
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.ActivityProjectListBinding
import com.project.survey.extend.smartDismiss
import com.project.survey.extend.toast
import com.project.survey.extend.toastLong
import com.project.survey.logic.event.Message
import com.project.survey.logic.viewmodel.ProjectViewModel
import com.project.survey.model.ProjectBean
import com.project.survey.ui.MainActivity
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.login.LoginActivity
import com.project.survey.ui.project.adapter.ProjectListAdapter
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.LogoutUtil
import com.project.survey.util.SPUtils
import com.project.survey.widget.decoration.HorDividerDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshListener
import java.util.Date

class ProjectListActivity : BaseBindingActivity<ActivityProjectListBinding>(), OnRefreshListener {

    companion object {
        fun start() {
            ActivityNavUtil.startActivity<ProjectListActivity> { }
        }
    }

    override fun getBinding(): ActivityProjectListBinding =
        ActivityProjectListBinding.inflate(layoutInflater)

    private val viewModel: ProjectViewModel by viewModels()
    private val adapter by lazy { ProjectListAdapter() }

    override fun initView() {
        immersionToolbar(mBinding.toolbar, "项目列表")
//        ImmersionBar.with(this)
//            .statusBarDarkFont(true)
//            .titleBarMarginTop(mBinding.ilToolBar.toolbar)
//            .init()
//        setSupportActionBar(mBinding.ilToolBar.toolbar)

//        mBinding.ilToolBar.toolbar.setBackgroundColor(0)
//        mBinding.ilToolBar.toolbar.setTitle("项目列表")
//        mBinding.ilToolBar.toolbar.setTitleTextColor(ContextCompat.getColor(this, R.color.text_color_main))
//        initTabLayout()
        // 搜索
        mBinding.searchWidget.setOnEditorSearchListener {
            // 搜索 it
        }

        mBinding.smartRefreshLayout.setOnRefreshListener(this)
        mBinding.recyclerView.addItemDecoration(HorDividerDecoration(this))
        mBinding.recyclerView.setAdapter(adapter)
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            toast(it)
            mBinding.smartRefreshLayout.smartDismiss()
        }
        viewModel.projectResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
            if (it.isNotEmpty()) {
                adapter.submitList(it)
            } else {
                adapter.setStateViewLayout(this, R.layout.layout_no_data)
            }
        }
        fetchData()
    }

    private fun fetchData() {
        viewModel.fetchProjectList()
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        fetchData()
    }

    override fun initListener() {
        adapter.setOnItemClickListener { adapter, _, position ->
            adapter.getItem(position)?.let { bean ->
                // 进入项目，设置当前项目
                SPUtils.put(SPConstants.CURRENT_PROJECT, bean)
                SPUtils.put(SPConstants.PROJECT_ID, bean.XMXX_ID)
                // 查询当前项目坐标系统
                viewModel.fetchCoordinateSystem({ coordinateSystemValue ->
                    SPUtils.put(
                        SPConstants.CURRENT_COORDINATE_SYSTEM_ID,
                        coordinateSystemValue.BD_ZBZHCS_ID
                    )
                    nav2Main(
                        bean,
                        CoordinateSystemUtil.importCoordSysString(coordinateSystemValue.ZBZHCS_ZHCS)
                    )
                }, {
                    SPUtils.put(SPConstants.CURRENT_COORDINATE_SYSTEM_ID, "")
                    nav2Main(bean)
                })

            }
        }
    }

    private fun nav2Main(
        bean: ProjectBean,
        coordinateSystem: CoordinateSystem = CoordinateSystem.newCoordinateSystem(this)
    ) {
        // 存储配置--当前项目
        ThreadPoolUtil.execute {
            //存储新的项目
            ProjectDb.getInstance().save(
                ProjectRecord(
                    bean.XMXX_ID,
                    bean.XMXX_NAME,
                    coordinateSystem,
                    Date().time
                ).apply {
                    author = bean.XMXX_CONTACTNAME
                }
            )

            var currentProjectConfig =
                KeyValueDb.getInstance().findData(KeyValueConstants.KEY_CURRENT_PROJECT_ID)
            if (currentProjectConfig != null) {
                currentProjectConfig.value = bean.XMXX_ID
            } else {
                currentProjectConfig =
                    KeyValueRecord(
                        KeyValueConstants.KEY_CURRENT_PROJECT_ID,
                        bean.XMXX_ID
                    )
            }
            KeyValueDb.getInstance().save(currentProjectConfig)
        }

        viewModel.msgEvent.postValue(Message(EventConstants.UPDATE_PROJECT))
        MainActivity.start()
        ActivityNavUtil.finishActivity<ProjectListActivity>()
    }

//    private fun initTabLayout() {
//        mBinding.tabLayout.setTabData(
//            arrayListOf(
//                TabEntity(
//                    "最近打开",
//                    R.drawable.ic_project_recently_selected,
//                    R.drawable.ic_project_recently_unselected
//                ),
//                TabEntity(
//                    "全部",
//                    R.drawable.ic_project_all_selected,
//                    R.drawable.ic_project_all_unselected
//                )
//            ),
//            this,
//            R.id.container,
//            arrayListOf(
//                ProjectListFragment.newInstance(ProjectListFragment.TYPE_RECENTLY),
//                ProjectListFragment.newInstance(ProjectListFragment.TYPE_ALL)
//            )
//        )
//    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_activity_project_list, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.menu_activity_project_list_exit) {
            MaterialDialog.Builder(this)
                .title("确定退出吗?")
                .negativeText("取消")
                .positiveText("退出")
                .onPositive { _, _ ->
                    LogoutUtil.clearSP()
                    ActivityNavUtil.startActivity<LoginActivity> { }
                    ActivityNavUtil.finishActivity<ProjectListActivity>()
//                    viewModel.msgEvent.postValue(Message(EventConstants.LOGIN_STATUS))
                }
                .show()
        }
        return super.onOptionsItemSelected(item)
    }

}